import 'dart:async';
import 'dart:collection';

import 'package:flutter/widgets.dart';
import 'package:flutter_map/plugin_api.dart';
import 'package:flutter_map_marker_popup/src/popup_builder.dart';
import 'package:flutter_map_marker_popup/src/popup_container/popup_container_mixin.dart';
import 'package:flutter_map_marker_popup/src/popup_snap.dart';
import 'package:flutter_map_marker_popup/src/popup_spec.dart';
import 'package:flutter_map_marker_popup/src/state/popup_event.dart';
import 'package:flutter_map_marker_popup/src/state/popup_state_impl.dart';

class SimplePopupContainer extends StatefulWidget {
  final FlutterMapState mapState;
  final PopupStateImpl popupStateImpl;
  final PopupBuilder popupBuilder;
  final PopupSnap snap;

  const SimplePopupContainer({
    super.key,
    required this.mapState,
    required this.popupStateImpl,
    required this.snap,
    required this.popupBuilder,
  });

  @override
  State<StatefulWidget> createState() => _SimplePopupContainerState();
}

class _SimplePopupContainerState extends State<SimplePopupContainer>
    with PopupContainerMixin {
  late Set<PopupSpec> _selectedPopupSpecs;

  late StreamSubscription<PopupEvent> _popupStateEventSubscription;

  @override
  FlutterMapState get mapState => widget.mapState;

  @override
  PopupStateImpl get popupStateImpl => widget.popupStateImpl;

  @override
  PopupSnap get snap => widget.snap;

  @override
  void initState() {
    super.initState();
    _popupStateEventSubscription =
        widget.popupStateImpl.stream.listen(handleEvent);
    _selectedPopupSpecs = LinkedHashSet.from(
      widget.popupStateImpl.selectedMarkers.map(PopupSpec.wrap),
    );
  }

  @override
  void didUpdateWidget(covariant SimplePopupContainer oldWidget) {
    if (oldWidget.popupStateImpl != widget.popupStateImpl) {
      _popupStateEventSubscription.cancel();
      _popupStateEventSubscription =
          widget.popupStateImpl.stream.listen(handleEvent);
      _selectedPopupSpecs
        ..clear()
        ..addAll(widget.popupStateImpl.selectedPopupSpecs);
    }
    super.didUpdateWidget(oldWidget);
  }

  @override
  void dispose() {
    _popupStateEventSubscription.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (_selectedPopupSpecs.isEmpty) return Container();

    return Stack(
      children: _selectedPopupSpecs
          .map((popupSpec) => inPosition(
                popupSpec,
                popupWithStateKeepAlive(popupSpec, widget.popupBuilder),
              ))
          .toList(),
    );
  }

  @override
  void showPopupsAlsoFor(
    List<PopupSpec> popupSpecs, {
    required bool disableAnimation,
  }) {
    setState(() {
      _selectedPopupSpecs.addAll(popupSpecs);
    });
  }

  @override
  void showPopupsOnlyFor(
    List<PopupSpec> popupSpecs, {
    required bool disableAnimation,
  }) {
    setState(() {
      _selectedPopupSpecs.clear();
      _selectedPopupSpecs.addAll(popupSpecs);
    });
  }

  @override
  void hideAllPopups({required bool disableAnimation}) {
    if (_selectedPopupSpecs.isNotEmpty) {
      setState(() {
        _selectedPopupSpecs.clear();
      });
    }
  }

  @override
  void hidePopupsOnlyFor(
    List<PopupSpec> popupSpecs, {
    required bool disableAnimation,
  }) {
    setState(() {
      _selectedPopupSpecs.removeAll(popupSpecs);
    });
  }
}
